<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


declare(strict_types=1);

namespace SAPModule\Controller;


use App\itnovum\openITCOCKPIT\Wizard\CreateService;
use App\itnovum\openITCOCKPIT\Wizard\UpdateHost;
use App\Model\Entity\Host;
use App\Model\Table\HostsTable;
use App\Model\Table\HosttemplatesTable;
use App\Model\Table\ServicesTable;
use App\Model\Table\ServicetemplatesTable;
use App\Model\Table\WizardAssignmentsTable;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use SAPModule\Form\SapForm;

/**
 * Class WizardsController
 * @package SAPModule\Controller
 */
class WizardsController extends AppController {
    /**
     * @param null $hostId
     */
    public function sap($hostId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var WizardAssignmentsTable $WizardAssignmentsTable */
        $WizardAssignmentsTable = TableRegistry::getTableLocator()->get('WizardAssignments');
        $wizards = $WizardAssignmentsTable->getAvailableWizards($this->PERMISSIONS);
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            if (!$HostsTable->existsById($hostId)) {
                throw new NotFoundException();
            }
            $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
            if (!$this->allowedByContainerId($host->getContainerIds())) {
                $this->render403();
                return;
            }
            $hostCustomVariables = $HostsTable->get($hostId, [
                'contain' => [
                    'Customvariables'
                ]
            ])->getCustomvariablesForCfg();

            $sysnr = '';
            if (!empty($hostCustomVariables['_SYSNR'])) {
                $sysnr = $hostCustomVariables['_SYSNR'];
            }
            $client = '';
            if (!empty($hostCustomVariables['_CLIENT'])) {
                $client = $hostCustomVariables['_CLIENT'];
            }
            $rfcuser = '';
            if (!empty($hostCustomVariables['_RFCUSER'])) {
                $rfcuser = $hostCustomVariables['_RFCUSER'];
            }
            $rfcpassword = '';
            if (!empty($hostCustomVariables['_RFCPASSWORD'])) {
                $rfcpassword = $hostCustomVariables['_RFCPASSWORD'];
            }
            $sid = '';
            if (!empty($hostCustomVariables['_SID'])) {
                $sid = $hostCustomVariables['_SID'];
            }
            $msgroup = '';
            if (!empty($hostCustomVariables['_MSGROUP'])) {
                $msgroup = $hostCustomVariables['_MSGROUP'];
            }
            $this->set('sysnr', $sysnr);
            $this->set('client', $client);
            $this->set('rfcuser', $rfcuser);
            $this->set('rfcpassword', $rfcpassword);
            $this->set('sid', $sid);
            $this->set('msgroup', $msgroup);

            //Return sap wizard data
            $servicetemplates = [];
            $wizardAssignments = [];
            $sapData = $wizards['sap'] ?? null;
            if ($sapData) {
                if ($WizardAssignmentsTable->existsByUuidAndTypeId($sapData['uuid'], $sapData['type_id'])) {
                    $wizardAssignments = Hash::merge($sapData, $WizardAssignmentsTable->getWizardByUuidForEdit($sapData['uuid']));
                }
            }
            if (!empty($wizardAssignments['servicetemplates']['_ids'])) {
                /** @var ServicetemplatesTable $ServicetemplatesTable */
                $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');
                $servicetemplates = $ServicetemplatesTable->getServicetemplatesFoWizardDeploy(
                    $wizardAssignments['servicetemplates']['_ids'],
                    $this->MY_RIGHTS
                );
            }
            $servicesNamesForExistCheck = $ServicesTable->getServiceNamesByHostIdForWizard($hostId);


            $this->set('servicetemplates', $servicetemplates);
            $this->set('servicesNamesForExistCheck', $servicesNamesForExistCheck);

            $this->viewBuilder()->setOption('serialize', [
                'servicetemplates',
                'servicesNamesForExistCheck',
                'sysnr',
                'client',
                'rfcuser',
                'rfcpassword',
                'sid',
                'msgroup'
            ]);
            return;
        }
        if ($this->request->is('post') && $this->isAngularJsRequest()) {
            $hostId = $this->request->getData('host_id', 0);
            if (!$HostsTable->existsById($hostId)) {
                throw new NotFoundException();
            }
            $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
            if (!$this->allowedByContainerId($host->getContainerIds())) {
                $this->render403();
                return;
            }
            $SapForm = new SapForm();
            $data = $this->request->getData(null, []);
            $SapForm->execute($data);

            if (!empty($SapForm->getErrors())) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $SapForm->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            //Wizard validation successfully - update host and create services
            $User = new User($this->getUser());

            /** @var HosttemplatesTable $HosttemplatesTable */
            $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');

            // Get host and host template for merge
            $host = $HostsTable->getHostForEdit($hostId);
            $hosttemplate = $HosttemplatesTable->getHosttemplateForDiff($host['Host']['hosttemplate_id']);

            //Update custom variables
            $hostCustomVariablestoCheck = [
                'SYSNR'       => 'sysnr',
                'CLIENT'      => 'client',
                'RFCUSER'     => 'rfcuser',
                'RFCPASSWORD' => 'rfcpassword',
                'SID'         => 'sid',
                'MSGROUP'     => 'msgroup'
            ];


            //Update host with changelog
            $hostEntity = UpdateHost::save(
                $host,
                $hosttemplate,
                $data,
                $hostCustomVariablestoCheck,
                $HostsTable,
                $User
            );

            if ($hostEntity instanceof Host) {
                //Host updated successfully - create services
                /** @var ServicetemplatesTable $ServicetemplatesTable */
                $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');

                // Save default SAP services
                CreateService::saveMany(
                    $hostEntity,
                    $data['services'],
                    $data,
                    [],
                    $HostsTable,
                    $HosttemplatesTable,
                    $ServicesTable,
                    $ServicetemplatesTable,
                    $User
                );
            }
        }
        return;
    }
}
