#!/bin/bash

# Class named 'SAPModule' for UPDATE.sh bash object emulation

# property
properties=()

# properties IDs (only placeholder with incrementing array position value)
name=0
disableFileLock=1
dbIniFile=2
anOtherProperty=3

# predefined properties
properties[name]='SAPModule'
# set 1 to disable file lock, this disables the uninstall() method and will run install() on every openitcockpit-update if the module folder exists
properties[disableFileLock]=1

# do not change this method!
SAPModule.property() {
  if [ "$2" == "=" ]; then
    properties[$1]=$3
  else
    echo ${properties[$1]}
  fi
}

# runs on every openitcockpit-update
SAPModule.initialize() {
  SAPModule.installIfNeeded
}

SAPModule.installIfNeeded() {
  if [ $(SAPModule.property disableFileLock) == 1 ]; then
    if [ $(system.moduleSrcDirExists $(SAPModule.property name)) == 1 ]; then
      SAPModule.install
    fi
  else
    if [ $(system.moduleSrcDirExists $(SAPModule.property name)) == 1 ]; then
      if [ "$(system.isLocked $(SAPModule.property name) 'installed')" != 1 ]; then
        SAPModule.install
      fi
    else
      if [ "$(system.isLocked $(SAPModule.property name) 'installed')" == 1 ]; then
        SAPModule.uninstall
      fi
    fi
  fi
}

SAPModule.install() {
  # insert custom code
  system.lock $(SAPModule.property name) "installed" $(SAPModule.property disableFileLock)
}

SAPModule.uninstall() {
  echo "I am the migrated postrm routine of $(SAPModule.property name)"
  # insert custom code
  system.unlock $(SAPModule.property name) "installed" $(SAPModule.property disableFileLock)
}

SAPModule.finish() {
  if [ $(system.moduleSrcDirExists $(SAPModule.property name)) == 1 ]; then
    # insert custom code
    echo "I run after every openitcockpit-update" >/dev/null
  fi
}
