<?php
// Copyright (C) <2015>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace SAPModule\Lib\InitialDatabase;

use App\Model\Table\CommandsTable;
use App\Model\Table\HosttemplatesTable;
use App\Model\Table\ServicetemplategroupsTable;
use App\Model\Table\ServicetemplatesTable;
use App\Model\Table\WizardAssignmentsTable;
use itnovum\openITCOCKPIT\InitialDatabase\Importer;

/**
 * Class TemplateImport
 * @package SAPModule\Lib\InitialDatabase
 */
class TemplateImport extends Importer {
    /**
     * @var CommandsTable
     */
    private $CommandsTable;

    /**
     * @var ServicetemplatesTable
     */
    private $ServicetemplatesTable;

    /**
     * @var HosttemplatesTable
     */
    private $HosttemplatesTable;

    /**
     * @var ServicetemplategroupsTable
     */
    private $ServicetemplategroupsTable;

    /**
     * @var WizardAssignmentsTable
     */
    private $WizardAssignmentsTable;

    /**
     * @var array
     */
    private $mapping = [];

    /**
     * TemplateImport constructor.
     * @param CommandsTable $CommandsTable
     * @param ServicetemplatesTable $ServicetemplatesTable
     * @param HosttemplatesTable $HosttemplatesTable
     * @param ServicetemplategroupsTable $ServicetemplategroupsTable
     * @param WizardAssignmentsTable $WizardAssignmentsTable
     */
    public function __construct(CommandsTable $CommandsTable, ServicetemplatesTable $ServicetemplatesTable, HosttemplatesTable $HosttemplatesTable, ServicetemplategroupsTable $ServicetemplategroupsTable, WizardAssignmentsTable $WizardAssignmentsTable) {
        $this->CommandsTable = $CommandsTable;
        $this->ServicetemplatesTable = $ServicetemplatesTable;
        $this->HosttemplatesTable = $HosttemplatesTable;
        $this->ServicetemplategroupsTable = $ServicetemplategroupsTable;
        $this->WizardAssignmentsTable = $WizardAssignmentsTable;
    }

    /**
     * @return bool
     */
    public function import() {
        $data = $this->getData();
        foreach ($data['Commands'] as $command) {
            if (isset($command['uuid']) && !$this->CommandsTable->existsByUuid($command['uuid'])) {
                $entity = $this->CommandsTable->newEntity($command);
                $this->CommandsTable->save($entity);
            }
        }
        $this->mapping['hosttemplates'] = [];
        foreach ($data['Hosttemplates'] as $hosttemplate) {
            if ($this->HosttemplatesTable->existsByUuid($hosttemplate['uuid'])) {
                $existingHosttemplate = $this->HosttemplatesTable->getHosttemplateByUuid($hosttemplate['uuid']);
                $this->mapping['hosttemplates'][$hosttemplate['uuid']] = $existingHosttemplate['Hosttemplate']['id'];
                //skip current template if it already exists
                continue;
            }
            if (isset($hosttemplate['uuid']) && !$this->HosttemplatesTable->existsByUuid($hosttemplate['uuid'])) {
                if (isset($hosttemplate['command_id']) && $this->CommandsTable->existsByUuid($hosttemplate['command_id'])) {
                    $command = $this->CommandsTable->getCommandByUuid($hosttemplate['command_id'], true, false)[0];
                    $hosttemplate['command_id'] = $command['id'];
                    if (!empty($hosttemplate['hosttemplatecommandargumentvalues']) && !empty($command['commandarguments'])) {
                        foreach ($hosttemplate['hosttemplatecommandargumentvalues'] as $templateArgumentKey => $templateArgumentValue) {
                            $hosttemplate['hosttemplatecommandargumentvalues'][$templateArgumentKey] = [
                                'commandargument_id' => $this->getCommandArgumentIdByName($templateArgumentValue['commandargument_id'], $command['commandarguments']),
                                'value'              => $templateArgumentValue['value'],
                            ];
                        }
                    }
                    $entity = $this->HosttemplatesTable->newEntity($hosttemplate);
                    $this->HosttemplatesTable->save($entity);
                    $this->mapping['hosttemplates'][$entity->get('uuid')] = $entity->get('id');
                }
            }
        }


        $this->mapping['servicetemplates'] = [];

        foreach ($data['Servicetemplates'] as $servicetemplate) {
            if ($this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                $existingServicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplate['uuid']);
                $this->mapping['servicetemplates'][$servicetemplate['uuid']] = $existingServicetemplate['Servicetemplate']['id'];
                //skip current template if it already exists
                continue;
            }
            if (isset($servicetemplate['uuid']) && !$this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                if (isset($servicetemplate['command_id']) && $this->CommandsTable->existsByUuid($servicetemplate['command_id'])) {
                    $command = $this->CommandsTable->getCommandByUuid($servicetemplate['command_id'], true, false)[0];
                    $servicetemplate['command_id'] = $command['id'];
                    if (!empty($servicetemplate['servicetemplatecommandargumentvalues']) && !empty($command['commandarguments'])) {
                        foreach ($servicetemplate['servicetemplatecommandargumentvalues'] as $templateArgumentKey => $templateArgumentValue) {
                            $servicetemplate['servicetemplatecommandargumentvalues'][$templateArgumentKey] = [
                                'commandargument_id' => $this->getCommandArgumentIdByName($templateArgumentValue['commandargument_id'], $command['commandarguments']),
                                'value'              => $templateArgumentValue['value'],
                            ];
                        }
                    }
                    $entity = $this->ServicetemplatesTable->newEntity($servicetemplate);
                    $this->ServicetemplatesTable->save($entity);
                    $this->mapping['servicetemplates'][$entity->get('uuid')] = $entity->get('id');
                }
            }
        }

        foreach ($data['Servicetemplategroups'] as $servicetemplategroup) {
            if ($this->ServicetemplategroupsTable->existsByUuid($servicetemplategroup['uuid'])) {
                //skip current template if its already exists
                continue;
            }

            if (!empty($this->mapping['servicetemplates'])) {
                foreach ($servicetemplategroup['servicetemplates'] as $key => $servicetemplate) {
                    $currIdAsUuid = $servicetemplate['id'];
                    if (isset($this->mapping['servicetemplates'][$currIdAsUuid])) {
                        $servicetemplategroup['servicetemplates']['_ids'][] = $this->mapping['servicetemplates'][$currIdAsUuid];
                    }

                }
            }

            $entity = $this->ServicetemplategroupsTable->newEntity($servicetemplategroup);
            $this->ServicetemplategroupsTable->save($entity);
        }


        if (!empty($data['Wizardassignment'])) {
            $wizardassignmentEntity['uuid'] = $data['Wizardassignment']['uuid'];
            $wizardassignmentEntity['type_id'] = $data['Wizardassignment']['type_id'];
            foreach ($data['Wizardassignment']['servicetemplates']['_ids'] as $servicetemplateUuid) {
                $servicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplateUuid);
                if (isset($servicetemplate['Servicetemplate']) && isset($servicetemplate['Servicetemplate']['id'])) {
                    $wizardassignmentEntity['servicetemplates']['_ids'][] = $servicetemplate['Servicetemplate']['id'];
                }
            }
            if (!$this->WizardAssignmentsTable->existsByUuidAndTypeId($wizardassignmentEntity['uuid'], $wizardassignmentEntity['type_id'])) {
                $entity = $this->WizardAssignmentsTable->newEntity($wizardassignmentEntity);
                $this->WizardAssignmentsTable->save($entity);
            } else {
                $existingAssignment = $this->WizardAssignmentsTable->getWizardByUuidForEdit($wizardassignmentEntity['uuid']);
                foreach (array_diff($wizardassignmentEntity['servicetemplates']['_ids'], $existingAssignment['servicetemplates']['_ids']) as $missingServicetemplateId) {
                    $existingAssignment['servicetemplates']['_ids'][] = $missingServicetemplateId;
                }
                $entity = $this->WizardAssignmentsTable->get($existingAssignment['id']);
                $entity = $this->WizardAssignmentsTable->patchEntity($entity, $existingAssignment);
                $this->WizardAssignmentsTable->save($entity);
            }
        }
        return true;
    }

    /**
     * @return array
     */
    public function getData() {
        return [
            'Commands'              => $this->getCommandsData(),
            'Servicetemplates'      => $this->getServicetemplatesData(),
            'Hosttemplates'         => $this->getHosttemplatesData(),
            'Servicetemplategroups' => $this->getServicetemplategroupsData(),
            'Wizardassignment'      => $this->getWizardAssignmentData()
        ];
    }

    /**
     * @return array
     */
    public function getCommandsData() {
        $data = [
            0 =>
                [
                    'name'             => 'check_sap_health',
                    'command_line'     => '$USER1$/check_sap_health --ashost $HOSTADDRESS$ --sysnr $_HOSTSYSNR$ --client $_HOSTCLIENT$ --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode $ARG1$ --warning $ARG2$ --critical $ARG3$',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'description'      => '',
                    'commandarguments' =>
                        [
                            0 =>
                                [
                                    'name'       => '$ARG1$',
                                    'human_name' => 'Mode',
                                ],
                            1 =>
                                [
                                    'name'       => '$ARG2$',
                                    'human_name' => 'Warning',
                                ],
                            2 =>
                                [
                                    'name'       => '$ARG3$',
                                    'human_name' => 'Critical',
                                ],
                        ],
                ],
            1 =>
                [
                    'name'             => 'check_sap_ws_checkitem',
                    'command_line'     => '$USER1$/check_sap_webservices.py --hostname $HOSTADDRESS$ --port 5$_HOSTSYSNR$13 --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode checkitem --item "$ARG1$" --warning $ARG2$ --critical $ARG3$',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'description'      => '',
                    'commandarguments' =>
                        [
                            0 =>
                                [
                                    'name'       => '$ARG1$',
                                    'human_name' => 'Item',
                                ],
                            1 =>
                                [
                                    'name'       => '$ARG2$',
                                    'human_name' => 'Warning - set it to na if you want to get the status from SAP',
                                ],
                            2 =>
                                [
                                    'name'       => '$ARG3$',
                                    'human_name' => 'Critical - set it to na if you want to get the status from SAP',
                                ],
                        ],
                ],
            2 =>
                [
                    'name'             => 'check_sap_ws_checksyslog',
                    'command_line'     => '$USER1$/check_sap_webservices.py --hostname $HOSTADDRESS$ --port 5$_HOSTSYSNR$13 --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode checksyslog --statearg $SERVICEDESC$',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => '0952414f-ad0c-4ccb-8e5f-0b4cd018dbba',
                    'description'      => '',
                    'commandarguments' =>
                        [
                        ],
                ],
            3 =>
                [
                    'name'             => 'check_sap_ws_checksyslog_regex',
                    'command_line'     => '$USER1$/check_sap_webservices.py --hostname $HOSTADDRESS$ --port 5$_HOSTSYSNR$13 --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode checksyslog --mode2 regex --item "$ARG1$" --statearg $SERVICEDESC$',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => '5583bde3-834d-4e56-b99a-4f871e384a8d',
                    'description'      => '',
                    'commandarguments' =>
                        [
                            0 =>
                                [
                                    'name'       => '$ARG1$',
                                    'human_name' => 'Regex to search in syslog',
                                ],
                        ],
                ],
            4 =>
                [
                    'name'             => 'check_sap_ws_getsyslog',
                    'command_line'     => '$USER1$/check_sap_webservices.py --hostname $HOSTADDRESS$ --port 5$_HOSTSYSNR$13 --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode getsyslog',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => 'a66e6c2f-0372-40fa-8bec-4f7b5053e305',
                    'description'      => '',
                    'commandarguments' =>
                        [
                        ],
                ],
            5 =>
                [
                    'name'             => 'check_sap_ws_gettree',
                    'command_line'     => '$USER1$/check_sap_webservices.py --hostname $HOSTADDRESS$ --port 5$_HOSTSYSNR$13 --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode gettree',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => '2d2ce02e-8d31-44bf-81ca-25e8fdf7fb4a',
                    'description'      => '',
                    'commandarguments' =>
                        [
                        ],
                ],
            6 =>
                [
                    'name'             => 'check_sap_ws_workprocesses',
                    'command_line'     => '$USER1$/check_sap_webservices.py --hostname $HOSTADDRESS$ --port 5$_HOSTSYSNR$13 --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode workprocess --item $ARG1$ --warning $ARG2$ --critical $ARG3$',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => '11788fb3-8c94-4ff2-b296-69abb71b52f7',
                    'description'      => '',
                    'commandarguments' =>
                        [
                            0 =>
                                [
                                    'name'       => '$ARG1$',
                                    'human_name' => 'Item',
                                ],
                            1 =>
                                [
                                    'name'       => '$ARG2$',
                                    'human_name' => 'Warning',
                                ],
                            2 =>
                                [
                                    'name'       => '$ARG3$',
                                    'human_name' => 'Critical',
                                ],
                        ],
                ],
            7 =>
                [
                    'name'             => 'check_sap_health_mte',
                    'command_line'     => '$USER1$/check_sap_health --ashost $HOSTADDRESS$ --sysnr $_HOSTSYSNR$ --client $_HOSTCLIENT$ --username $_HOSTRFCUSER$ --password \'$_HOSTRFCPASSWORD$\' --mode ccms-mte-check --name \'$ARG1$\' --name2 \'$ARG2$\' --name3 \'$ARG3$\' --separator \'#\' --regexp --timeout 60 --warning $ARG4$ --critical $ARG5$',
                    'command_type'     => '1',
                    'human_args'       => null,
                    'uuid'             => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'description'      => '',
                    'commandarguments' =>
                        [
                            0 =>
                                [
                                    'name'       => '$ARG1$',
                                    'human_name' => 'Monitor-Set',
                                ],
                            1 =>
                                [
                                    'name'       => '$ARG2$',
                                    'human_name' => 'Monitor',
                                ],
                            2 =>
                                [
                                    'name'       => '$ARG3$',
                                    'human_name' => 'MTE',
                                ],
                            3 =>
                                [
                                    'name'       => '$ARG4$',
                                    'human_name' => 'Warning',
                                ],
                            4 =>
                                [
                                    'name'       => '$ARG5$',
                                    'human_name' => 'Critical',
                                ],
                        ],
                ],
        ];
        return $data;
    }

    /**
     * @return array
     */
    public function getHosttemplatesData() {
        $data = [
            0 =>
                [
                    'uuid'                              => 'be464ee4-5739-4a9b-beb8-3525c8f80afc',
                    'name'                              => 'SAP Host',
                    'description'                       => '',
                    'hosttemplatetype_id'               => '1',
                    'command_id'                        => '5a538ebc-03de-4ce6-8e32-665b841abde3',
                    'check_command_args'                => '',
                    'eventhandler_command_id'           => '0',
                    'timeperiod_id'                     => '0',
                    'check_interval'                    => '7200',
                    'retry_interval'                    => '60',
                    'max_check_attempts'                => '3',
                    'first_notification_delay'          => '0',
                    'notification_interval'             => '7200',
                    'notify_on_down'                    => '1',
                    'notify_on_unreachable'             => '0',
                    'notify_on_recovery'                => '1',
                    'notify_on_flapping'                => '0',
                    'notify_on_downtime'                => '0',
                    'flap_detection_enabled'            => '0',
                    'flap_detection_on_up'              => '0',
                    'flap_detection_on_down'            => '0',
                    'flap_detection_on_unreachable'     => '0',
                    'low_flap_threshold'                => '0',
                    'high_flap_threshold'               => '0',
                    'process_performance_data'          => '0',
                    'freshness_checks_enabled'          => '0',
                    'freshness_threshold'               => '0',
                    'passive_checks_enabled'            => '0',
                    'event_handler_enabled'             => '0',
                    'active_checks_enabled'             => '1',
                    'retain_status_information'         => '0',
                    'retain_nonstatus_information'      => '0',
                    'notifications_enabled'             => '1',
                    'notes'                             => '',
                    'priority'                          => '1',
                    'check_period_id'                   => '1',
                    'notify_period_id'                  => '1',
                    'tags'                              => '',
                    'container_id'                      => '1',
                    'host_url'                          => '',
                    'created'                           => '2015-01-05 15:22:21',
                    'modified'                          => '2015-01-05 15:22:21',
                    'hosttemplatecommandargumentvalues' =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => '3000.0,80%',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '5000.0,100%',
                                ],
                        ],
                    'customvariables'                   =>
                        [
                            0 =>
                                [
                                    'objecttype_id' => '512',
                                    'name'          => 'SYSNR',
                                    'value'         => '00',
                                    'password'      => 0,
                                ],
                            1 =>
                                [
                                    'objecttype_id' => '512',
                                    'name'          => 'SID',
                                    'value'         => 'XYZ',
                                    'password'      => 0,
                                ],
                            2 =>
                                [
                                    'objecttype_id' => '512',
                                    'name'          => 'CLIENT',
                                    'value'         => '000',
                                    'password'      => 0,
                                ],
                            3 =>
                                [
                                    'objecttype_id' => '512',
                                    'name'          => 'RFCUSER',
                                    'value'         => 'USERX',
                                    'password'      => 0,
                                ],
                            4 =>
                                [
                                    'objecttype_id' => '512',
                                    'name'          => 'RFCPASSWORD',
                                    'value'         => 'PASSY',
                                    'password'      => 1,
                                ],
                            5 =>
                                [
                                    'objecttype_id' => '512',
                                    'name'          => 'MSGROUP',
                                    'value'         => 'XXXYYYZZZ',
                                    'password'      => 0,
                                ],
                        ],
                    'hostgroups'                        =>
                        [
                        ],
                    'contactgroups'                     =>
                        [
                        ],
                    'contacts'                          =>
                        [
                            '_ids' =>
                                [
                                    0 => '1',
                                ],
                        ],
                ],
        ];
        return $data;
    }

    /**
     * @return array
     */
    public function getServicetemplatesData() {
        $data = [
            0  =>
                [
                    'uuid'                                      => '0a362c3d-492d-4a70-835c-5e906abe8fd8',
                    'template_name'                             => 'SAP_DATAPICKUP',
                    'name'                                      => 'SAP Datapickup',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '2d2ce02e-8d31-44bf-81ca-25e8fdf7fb4a',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:27:42',
                    'modified'                                  => '2020-09-08 11:27:42',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            1  =>
                [
                    'uuid'                                      => 'bb78bb0a-8f73-4872-9df3-f04a17ef87c4',
                    'template_name'                             => 'SAP_DATAPICKUP_SYSLOG',
                    'name'                                      => 'SAP Datapickup Syslog',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'a66e6c2f-0372-40fa-8bec-4f7b5053e305',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:27:42',
                    'modified'                                  => '2020-09-08 11:27:42',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            2  =>
                [
                    'uuid'                                      => '7c80700f-cec6-453e-8620-c5e27216fb77',
                    'template_name'                             => 'SAP_SYSLOG',
                    'name'                                      => 'SAP Syslog',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '0952414f-ad0c-4ccb-8e5f-0b4cd018dbba',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:27:42',
                    'modified'                                  => '2020-09-08 11:27:42',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            3  =>
                [
                    'uuid'                                      => '19ce6706-ac6e-413f-9b1d-8abbdc2cd801',
                    'template_name'                             => 'SAP_SYSLOG_REGEX',
                    'name'                                      => 'SAP Syslog Search',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '5583bde3-834d-4e56-b99a-4f871e384a8d',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:27:42',
                    'modified'                                  => '2020-09-08 11:27:42',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            4  =>
                [
                    'uuid'                                      => 'ded51ee1-62aa-4e9b-a75e-7f829eaff9e5',
                    'template_name'                             => 'SAP_WORKPROCESS_BTC_WAIT',
                    'name'                                      => 'SAP Workprocess BTC Wait',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '11788fb3-8c94-4ff2-b296-69abb71b52f7',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:30:38',
                    'modified'                                  => '2020-09-08 11:30:38',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'BTC',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '2',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '1',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            5  =>
                [
                    'uuid'                                      => '68ad675c-389e-4a43-b823-06452c2e7ab9',
                    'template_name'                             => 'SAP_WORKPROCESS_DIA_WAIT',
                    'name'                                      => 'SAP Workprocess DIA Wait',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '11788fb3-8c94-4ff2-b296-69abb71b52f7',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:30:38',
                    'modified'                                  => '2020-09-08 11:30:38',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'DIA',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '2',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '1',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            6  =>
                [
                    'uuid'                                      => 'ece23947-1298-4bc8-811e-b935475c8509',
                    'template_name'                             => 'SAP_WORKPROCESS_UPD_WAIT',
                    'name'                                      => 'SAP Workprocess UPD Wait',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '11788fb3-8c94-4ff2-b296-69abb71b52f7',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:30:38',
                    'modified'                                  => '2020-09-08 11:30:38',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'UPD',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '2',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '1',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            7  =>
                [
                    'uuid'                                      => '0239aed8-11a5-4aaf-9464-285c5e97fc50',
                    'template_name'                             => 'SAP_ABORTEDJOBS_LAST30MINUTES',
                    'name'                                      => 'SAP Aborted Jobs Last 30 Minutes',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:32:43',
                    'modified'                                  => '2020-09-08 11:32:43',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'failed-jobs --regexp --name .* --lookback 1800 --selectedperfdata "no-perfdata" --negate "no finished jobs were found"=ok',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '10',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '20',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            8  =>
                [
                    'uuid'                                      => 'bdd3e3bf-58fe-4979-bc40-49982bb5f0e0',
                    'template_name'                             => 'SAP_BUFFERS_CALENDAR_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers Calendar DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Calendar##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            9  =>
                [
                    'uuid'                                      => '27fc931b-2e70-4b99-a58f-f7080ea5ccdd',
                    'template_name'                             => 'SAP_BUFFERS_CALENDAR_SPACEUSED',
                    'name'                                      => 'SAP Buffers Calendar SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Calendar##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            10 =>
                [
                    'uuid'                                      => '4f51d6a0-3b27-4cf7-b2c0-5b282fe425d3',
                    'template_name'                             => 'SAP_BUFFERS_CUA_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers CUA DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##CUA##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            11 =>
                [
                    'uuid'                                      => '98aca158-a75a-4ae4-9b4f-469bd0fb86f0',
                    'template_name'                             => 'SAP_BUFFERS_CUA_SPACEUSED',
                    'name'                                      => 'SAP Buffers CUA SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##CUA##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            12 =>
                [
                    'uuid'                                      => '761c0350-307b-4ae5-986f-13f153ecfcca',
                    'template_name'                             => 'SAP_BUFFERS_EXPIMPSHM_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers Exp Imp SHM DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Exp./ Imp. SHM##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            13 =>
                [
                    'uuid'                                      => 'c1a248c2-9e2e-4107-8de9-2003f98fc643',
                    'template_name'                             => 'SAP_BUFFERS_EXPIMPSHM_SPACEUSED',
                    'name'                                      => 'SAP Buffers Exp Imp SHMSpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Exp./ Imp. SHM##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            14 =>
                [
                    'uuid'                                      => '8cc59b1f-37e6-4db0-9836-75a77e6add73',
                    'template_name'                             => 'SAP_BUFFERS_EXPORT/IMPORT_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers Export/import DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Export/import##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            15 =>
                [
                    'uuid'                                      => '88f71a5c-6e28-47f9-8187-65816e27baf5',
                    'template_name'                             => 'SAP_BUFFERS_EXPORT/IMPORT_SPACEUSED',
                    'name'                                      => 'SAP Buffers Export/import SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Export/import##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            16 =>
                [
                    'uuid'                                      => 'eebd9d2e-b05b-4715-8e96-5979a969e83e',
                    'template_name'                             => 'SAP_BUFFERS_FIELDDESCRIPTION_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers FieldDescription DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##FieldDescription##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            17 =>
                [
                    'uuid'                                      => '1ba438e8-7947-4f8e-bc1f-2738c379c7fc',
                    'template_name'                             => 'SAP_BUFFERS_FIELDDESCRIPTION_SPACEUSED',
                    'name'                                      => 'SAP Buffers FieldDescription SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##FieldDescription##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            18 =>
                [
                    'uuid'                                      => 'c8c5ae1d-4df2-4c6f-adf4-dbaa02cb5d76',
                    'template_name'                             => 'SAP_BUFFERS_INITIALRECORDS_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers InitialRecords DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##InitialRecords##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            19 =>
                [
                    'uuid'                                      => '54179f08-2469-4f66-9806-510c42314f75',
                    'template_name'                             => 'SAP_BUFFERS_INITIALRECORDS_SPACEUSED',
                    'name'                                      => 'SAP Buffers InitialRecords SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##InitialRecords##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            20 =>
                [
                    'uuid'                                      => 'a5949e1c-0911-4dea-a58d-081cb54354bf',
                    'template_name'                             => 'SAP_BUFFERS_NUMBERRANGE_FREEENTRIES',
                    'name'                                      => 'SAP Buffers NumberRange FreeEntries',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3BasisSystem##Buffers##NumberRange##FreeEntries',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            21 =>
                [
                    'uuid'                                      => 'b77ee087-4746-4293-b2ed-93469e12463c',
                    'template_name'                             => 'SAP_BUFFERS_OTR_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers OTR DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##OTR##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            22 =>
                [
                    'uuid'                                      => '3688e74e-6e91-4a78-97f0-1fd05e12d14d',
                    'template_name'                             => 'SAP_BUFFERS_OTR_SPACEUSED',
                    'name'                                      => 'SAP Buffers OTR SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##OTR##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            23 =>
                [
                    'uuid'                                      => 'b2a9c408-6453-4045-8e20-3f5b7efbf084',
                    'template_name'                             => 'SAP_BUFFERS_PROGRAM_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers Program DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Program##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            24 =>
                [
                    'uuid'                                      => '1960f0be-f1bc-4c93-b772-090c8f458d58',
                    'template_name'                             => 'SAP_BUFFERS_PROGRAM_SPACEUSED',
                    'name'                                      => 'SAP Buffers Program SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Program##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            25 =>
                [
                    'uuid'                                      => '257a88e8-645b-4a67-8cb7-ac3fc6659197',
                    'template_name'                             => 'SAP_BUFFERS_SCREEN_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers Screen DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Screen##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            26 =>
                [
                    'uuid'                                      => 'da7ab213-b175-4719-ad89-fe15316af3a3',
                    'template_name'                             => 'SAP_BUFFERS_SCREEN_SPACEUSED',
                    'name'                                      => 'SAP Buffers Screen SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##Screen##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            27 =>
                [
                    'uuid'                                      => '30293841-24eb-41d9-81c6-f0ffb00c3646',
                    'template_name'                             => 'SAP_BUFFERS_SHORTNAMETAB_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers ShortNameTAB DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##ShortNameTAB##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            28 =>
                [
                    'uuid'                                      => '5cb52931-3197-43dd-8454-5ccb6d3a7c8c',
                    'template_name'                             => 'SAP_BUFFERS_SHORTNAMETAB_SPACEUSED',
                    'name'                                      => 'SAP Buffers ShortNameTAB SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##ShortNameTAB##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            29 =>
                [
                    'uuid'                                      => '4479a0d2-ffae-44b9-b928-4d989898bce7',
                    'template_name'                             => 'SAP_BUFFERS_TABLEBUFFER_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers TableBuffer DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##TableBuffer##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            30 =>
                [
                    'uuid'                                      => '7288efda-7dbe-4119-ade8-8038151519de',
                    'template_name'                             => 'SAP_BUFFERS_TABLEBUFFER_SPACEUSED',
                    'name'                                      => 'SAP Buffers TableBuffer SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##TableBuffer##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            31 =>
                [
                    'uuid'                                      => 'a788168e-5da2-4991-a50b-4ec03af75f2c',
                    'template_name'                             => 'SAP_BUFFERS_TABLEDEFINITION_DIRECTORYUSED',
                    'name'                                      => 'SAP Buffers TableDefinition DirectoryUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##TableDefinition##DirectoryUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            32 =>
                [
                    'uuid'                                      => 'd7f66176-04d1-4316-8a72-396e8011a199',
                    'template_name'                             => 'SAP_BUFFERS_TABLEDEFINITION_SPACEUSED',
                    'name'                                      => 'SAP Buffers TableDefinition SpaceUsed',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Buffers##TableDefinition##SpaceUsed',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            33 =>
                [
                    'uuid'                                      => '32cf14dc-060e-48cb-a4ad-b51a34b26dae',
                    'template_name'                             => 'SAP_DIALOG_DBREQUESTTIME',
                    'name'                                      => 'SAP Dialog DBRequestTime',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##DBRequestTime',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            34 =>
                [
                    'uuid'                                      => '6fe8cbb0-8234-415c-aabb-723d11b5ecab',
                    'template_name'                             => 'SAP_DIALOG_ERRORFREQINWPDIA',
                    'name'                                      => 'SAP Dialog ErrorFreqInWpDIA',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##ErrorFreqInWpDIA',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            35 =>
                [
                    'uuid'                                      => '63e968db-61c5-452b-a498-6c65cee164e0',
                    'template_name'                             => 'SAP_DIALOG_ERRORSINWPDIA',
                    'name'                                      => 'SAP Dialog ErrorsInWpDIA',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##ErrorsInWpDIA',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            36 =>
                [
                    'uuid'                                      => '64915c2c-5b83-468a-a99c-103e966ecc6e',
                    'template_name'                             => 'SAP_DIALOG_FRONTENDRESPONSETIME',
                    'name'                                      => 'SAP Dialog FrontendResponseTime',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##FrontendResponseTime',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            37 =>
                [
                    'uuid'                                      => 'de406bf3-049d-4498-95fe-0936b2c75a20',
                    'template_name'                             => 'SAP_DIALOG_QUEUETIME',
                    'name'                                      => 'SAP Dialog QueueTime',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##QueueTime',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            38 =>
                [
                    'uuid'                                      => '7f6cdf7f-6471-4f51-843f-560a91485553',
                    'template_name'                             => 'SAP_DIALOG_RESPONSETIME',
                    'name'                                      => 'SAP Dialog ResponseTime',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##ResponseTime',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            39 =>
                [
                    'uuid'                                      => '6094ee60-e6b7-4158-afc7-78d1160ded84',
                    'template_name'                             => 'SAP_DIALOG_RESPONSETIMEDIALOG',
                    'name'                                      => 'SAP Dialog ResponseTimeDialog',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##ResponseTimeDialog',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            40 =>
                [
                    'uuid'                                      => 'a78f65c2-722c-428f-806c-4caf9cee9655',
                    'template_name'                             => 'SAP_DIALOG_RESPONSETIMEDIALOGRFC',
                    'name'                                      => 'SAP Dialog ResponseTimeDialogRFC',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##ResponseTimeDialogRFC',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            41 =>
                [
                    'uuid'                                      => 'ec121498-9e16-4245-9937-b95864d41c07',
                    'template_name'                             => 'SAP_DIALOG_USERSLOGGEDIN',
                    'name'                                      => 'SAP Dialog UsersLoggedIn',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##UsersLoggedIn',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            42 =>
                [
                    'uuid'                                      => '324b5bf2-54da-4be5-bb40-2bec79b91ada',
                    'template_name'                             => 'SAP_DIALOG_UTILISATION',
                    'name'                                      => 'SAP Dialog Utilisation',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##Utilisation',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            43 =>
                [
                    'uuid'                                      => '2de342c4-56bc-4985-b63e-206de62c6aef',
                    'template_name'                             => 'SAP_DIALOG_LONGRUNNERS',
                    'name'                                      => 'SAP Dialog LongRunners',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##LongRunners',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            44 =>
                [
                    'uuid'                                      => 'b16d0a14-bb21-4f8f-abb0-ef0234f890fe',
                    'template_name'                             => 'SAP_ENQUEUE_SERVER_UTILISATION_GRANULE_ARGUMENTS',
                    'name'                                      => 'SAP Enqueue Server Utilisation Granule Arguments',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Enqueue',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Enqueue#Enqueue Server#Granule Arguments Actual Utilisation',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            45 =>
                [
                    'uuid'                                      => 'f3dc4237-2034-4629-9df5-422735de9fd9',
                    'template_name'                             => 'SAP_ENQUEUE_SERVER_UTILISATION_GRANULE_ENTRIES',
                    'name'                                      => 'SAP Enqueue Server Utilisation Granule Entries',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Enqueue',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Enqueue#Enqueue Server#Granule Entries Actual Utilisation',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            46 =>
                [
                    'uuid'                                      => '3e53f345-3aef-4cb1-9bba-f755ab22cb77',
                    'template_name'                             => 'SAP_ENQUEUE_SERVER_UTILISATION_OWNER_NAMES',
                    'name'                                      => 'SAP Enqueue Server Utilisation Owner Names',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Enqueue',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Enqueue#Enqueue Server#Owner Names Actual Utilisation',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            47 =>
                [
                    'uuid'                                      => 'd80e9b1e-d1a0-4531-a39a-91677ceb4076',
                    'template_name'                             => 'SAP_EXCEEDEDJOBS_LAST30MINUTES',
                    'name'                                      => 'SAP Exceeded Jobs Last 30 Minutes',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:32:43',
                    'modified'                                  => '2020-09-08 11:32:43',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'exceeded-failed-jobs --regexp --name .* --lookback 1800 --selectedperfdata "no-perfdata"',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '3600',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '3600',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            48 =>
                [
                    'uuid'                                      => '31206347-a022-42e8-b427-fb2c83ad2126',
                    'template_name'                             => 'SAP_FAILEDIDOCS_LAST30MINUTES',
                    'name'                                      => 'SAP Failed IDocs Last 30 Minutes',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:32:43',
                    'modified'                                  => '2020-09-08 11:32:43',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'failed-idocs --lookback 1800 --report short',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '0',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '0',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            49 =>
                [
                    'uuid'                                      => '4fba0c56-8903-4bcc-897a-c4e36d1191a0',
                    'template_name'                             => 'SAP_FAILEDUPDATES_LAST30MINUTES',
                    'name'                                      => 'SAP Failed Updates Last 30 Minutes',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:32:43',
                    'modified'                                  => '2020-09-08 11:32:43',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'failed-updates --lookback 1800',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '10',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '20',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            50 =>
                [
                    'uuid'                                      => '6f0023a9-e35f-4805-aa4a-f7916c434552',
                    'template_name'                             => 'SAP_R3ABAP_SHORTDUMPSFREQUENCY',
                    'name'                                      => 'SAP R3Abap Shortdumps Frequency',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Abap##Shortdumps Frequency',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            51 =>
                [
                    'uuid'                                      => '5fd3066b-5927-4a77-b626-f0e8812827c5',
                    'template_name'                             => 'SAP_R3SYSLOG_SYSLOGFREQUENCY',
                    'name'                                      => 'SAP R3Syslog Syslog Frequency',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Syslog##SyslogFreq',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            52 =>
                [
                    'uuid'                                      => '7aacb951-6bb3-444f-bccf-1106701873ad',
                    'template_name'                             => 'SAP_RFC_CONNECTION',
                    'name'                                      => 'SAP RFC-Connection',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:56:54',
                    'modified'                                  => '2020-09-08 11:56:54',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'connection-time',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '2',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '5',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            53 =>
                [
                    'uuid'                                      => '2bcf28e8-bdf8-40aa-8732-7bbb6789803a',
                    'template_name'                             => 'SAP_SHORTDUMPS_LAST30MINUTES',
                    'name'                                      => 'SAP Shortdumps Last 30 Minutes',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '314d2dea-9f85-4ae1-a37a-3be08ee99cad',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:32:43',
                    'modified'                                  => '2020-09-08 11:32:43',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'shortdumps-count --lookback 1800',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '10',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '20',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            54 =>
                [
                    'uuid'                                      => '6ef4882c-1e32-4935-9345-696b80da3081',
                    'template_name'                             => 'SAP_SPOOLSERVICE_UTILISATION',
                    'name'                                      => 'SAP SpoolService Utilisation',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Spool##SpoolService##Utilisation',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '90',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '95',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            55 =>
                [
                    'uuid'                                      => '87121e19-6fc0-45cc-8588-23dcf4553f1a',
                    'template_name'                             => 'SAP_UPDATE_ABAPERRORINUPDATE',
                    'name'                                      => 'SAP Update Abap Error In Update',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'Update##Update##AbapErrorInUpdate',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            56 =>
                [
                    'uuid'                                      => 'fcc0153b-c05f-4d76-a707-898e77ee28d0',
                    'template_name'                             => 'SAP_DIALOG_PRIVMODE_UTILISATION',
                    'name'                                      => 'SAP Dialog PrivMode Utilisation',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Dialog##PrivMode Utilisation',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            57 =>
                [
                    'uuid'                                      => '03d28e08-1109-4032-8708-34ee2aff0040',
                    'template_name'                             => 'SAP_R3SERVICES_MEMORYMANAGMENT_ESACT',
                    'name'                                      => 'SAP Extended Memory Utilisation EsAct',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3BasisSystem##MemoryManagement##R3MemMgmtResources##EsAct',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            58 =>
                [
                    'uuid'                                      => '4eae2911-1371-4e61-b6d1-61139748cb08',
                    'template_name'                             => 'SAP_R3SERVICES_MEMORYMANAGMENT_HEAPACT',
                    'name'                                      => 'SAP HEAP Memory Utilisation HeapAct',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3BasisSystem##MemoryManagement##R3MemMgmtResources##HeapAct',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            59 =>
                [
                    'uuid'                                      => 'dddb8f26-1058-4bd5-80b1-38beb0e61ced',
                    'template_name'                             => 'SAP_SPOOLNUMBERS_USEDNUMBERS',
                    'name'                                      => 'SAP SpoolNumbers UsedNumbers',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Technical Expert Monitors',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'All Monitoring Contexts',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Spool#SpoolNumbers#UsedNumbers',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            60 =>
                [
                    'uuid'                                      => '619b14c4-2ce2-4161-b4be-6c4d5f37fbb7',
                    'template_name'                             => 'SAP_BACKGROUND_SYSTEM_WIDE_QUEUE_LENGTH',
                    'name'                                      => 'SAP Background System Wide Queue Length',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Background Processing',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Background#BackgroundService#SystemWideQueueLength',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            61 =>
                [
                    'uuid'                                      => 'aae243fa-e3d7-4978-9629-f6eb82295b08',
                    'template_name'                             => 'SAP_CONCURRENT_USERS_ALL_CLIENTS',
                    'name'                                      => 'SAP Concurrent Users all Clients',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Entire System',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'System Configuration#Concurrent Users#Concurrent Users \\(all Clients\\)',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            62 =>
                [
                    'uuid'                                      => 'ffbd4262-0bc6-45c7-ad7d-5e1f8df00355',
                    'template_name'                             => 'SAP_ENQUEUE_REQUESTS',
                    'name'                                      => 'SAP Enqueue Requests',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:26:07',
                    'modified'                                  => '2021-03-25 13:26:07',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Enqueue',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Enqueue#Enqueue Server#Enqueue Requests',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            63 =>
                [
                    'uuid'                                      => 'b8dea6be-1049-4225-8dc6-2a34e4608db4',
                    'template_name'                             => 'SAP_ENQUEUE_REQUEST_REJECTS',
                    'name'                                      => 'SAP Enqueue Request Rejects',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:26:07',
                    'modified'                                  => '2021-03-25 13:26:07',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Monitor Templates',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'Enqueue',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Enqueue#Enqueue Server#Enqueue Request Rejects',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            64 =>
                [
                    'uuid'                                      => 'f8400b9a-7dbe-4724-9426-211bfe74c711',
                    'template_name'                             => 'SAP_R3SERVICES_GATEWAY_CONNECTIONS',
                    'name'                                      => 'SAP Gateway Connections',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:35:21',
                    'modified'                                  => '2021-03-25 13:35:21',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Gateway_Summary##Gateway_Reader##Gateway_Connections',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '95',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '98',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            65 =>
                [
                    'uuid'                                      => 'ca3fd6e5-11e5-4991-b036-64107a81c258',
                    'template_name'                             => 'SAP_R3SERVICES_ICM_NOOFTHREADS',
                    'name'                                      => 'SAP ICM - no of Threads',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:35:21',
                    'modified'                                  => '2021-03-25 13:35:21',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##ICM##General##NoOfThreads',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            66 =>
                [
                    'uuid'                                      => '871cb131-d3a2-43fa-960e-338e8d05fa59',
                    'template_name'                             => 'SAP_R3SERVICES_MEMORYMANAGMENT_ESPEAK',
                    'name'                                      => 'SAP Extended Memory peak EsPeak',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3BasisSystem##MemoryManagement##R3MemMgmtResources##EsPeak',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            67 =>
                [
                    'uuid'                                      => '79ade959-5193-490b-b624-9f7e3503c529',
                    'template_name'                             => 'SAP_R3SERVICES_MEMORYMANAGMENT_HEAPPEAK',
                    'name'                                      => 'SAP HEAP Memory peak HeapPeak',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '60',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2020-09-08 11:34:19',
                    'modified'                                  => '2020-09-08 11:34:19',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3BasisSystem##MemoryManagement##R3MemMgmtResources##HeapPeak',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'na',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'na',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            68 =>
                [
                    'uuid'                                      => 'b70afa65-a2a0-4423-b9ba-a857fb4b97ba',
                    'template_name'                             => 'SAP_R3SERVICES_UPDATE_PERFORMANCE_U1_RESPONSETIME',
                    'name'                                      => 'SAP Update1 Responsetime',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:44:18',
                    'modified'                                  => '2021-03-25 13:44:18',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Update##PerformanceU1##ResponseTime',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '30000',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '50000',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            69 =>
                [
                    'uuid'                                      => '0727f21b-6307-4d82-aa46-aa4bbc91ebdb',
                    'template_name'                             => 'SAP_R3SERVICES_UPDATE_PERFORMANCE_U2_RESPONSETIME',
                    'name'                                      => 'SAP Update2 Responsetime',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:44:18',
                    'modified'                                  => '2021-03-25 13:44:18',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Update##PerformanceU2##ResponseTime',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '30000',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '50000',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            70 =>
                [
                    'uuid'                                      => 'e97caf28-50e5-42c0-aef1-789353b89aba',
                    'template_name'                             => 'SAP_R3SERVICES_UPDATE_PERFORMANCE_U1_UTILISATION',
                    'name'                                      => 'SAP Update1 Utilisation',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:44:18',
                    'modified'                                  => '2021-03-25 13:44:18',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Update##PerformanceU1##Utilisation',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '95',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '98',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            71 =>
                [
                    'uuid'                                      => '8976fedf-96af-4b14-a571-9fd255503aec',
                    'template_name'                             => 'SAP_R3SERVICES_UPDATE_PERFORMANCE_U2_UTILISATION',
                    'name'                                      => 'SAP Update2 Utilisation',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => '247bd1c7-b37b-4a77-8542-ec8cffd6e8c5',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '300',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 13:44:18',
                    'modified'                                  => '2021-03-25 13:44:18',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'R3Services##Update##PerformanceU2##Utilisation',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => '95',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => '98',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            72 =>
                [
                    'uuid'                                      => 'ea318c60-92bc-418f-b3a5-59687f200b3e',
                    'template_name'                             => 'SAP_TRFC_QRFC_CALLS_COMMUNICATION_ERRORS',
                    'name'                                      => 'SAP tRFC - CPICERR',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '120',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 14:17:48',
                    'modified'                                  => '2021-03-25 14:17:48',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Technical Expert Monitors',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'All Monitoring Contexts',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Transactional RFC and Queued RFC#ARFCSSTATE: Outbound tRFC Calls#Calls w/Communication Errors - CPICERR',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            73 =>
                [
                    'uuid'                                      => '44c90da2-4f0d-4390-8c0f-f078d275810e',
                    'template_name'                             => 'SAP_TRFC_QRFC_CALLS_EXECUTION_ERRORS',
                    'name'                                      => 'SAP tRFC - SYSFAIL',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '120',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 14:17:48',
                    'modified'                                  => '2021-03-25 14:17:48',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Technical Expert Monitors',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'All Monitoring Contexts',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Transactional RFC and Queued RFC#ARFCSSTATE: Outbound tRFC Calls#Calls w/Execution Errors - SYSFAIL',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            74 =>
                [
                    'uuid'                                      => 'c58f0349-2d0e-4eec-9122-cdc0d5d6c9a0',
                    'template_name'                             => 'SAP_TRFC_QRFC_CALLS_SERVER_RESOURCES',
                    'name'                                      => 'SAP tRFC - SYSLOAD',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '120',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 14:17:48',
                    'modified'                                  => '2021-03-25 14:17:48',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Technical Expert Monitors',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'All Monitoring Contexts',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Transactional RFC and Queued RFC#ARFCSSTATE: Outbound tRFC Calls#Calls w/o RFC Server Resources - SYSLOAD',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            75 =>
                [
                    'uuid'                                      => 'a50fe427-42e0-49d0-89a1-a0f40a1b33ea',
                    'template_name'                             => 'SAP_TRFC_QRFC_CALLS_INBOUND_BLOCKED_QUEUES',
                    'name'                                      => 'SAP Inbound qRFC',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '120',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 14:17:48',
                    'modified'                                  => '2021-03-25 14:17:48',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Technical Expert Monitors',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'All Monitoring Contexts',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Transactional RFC and Queued RFC#(?\!.*Outbound.*).*#Blocked queues: Client $_HOSTCLIENT$',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
            76 =>
                [
                    'uuid'                                      => '6560c534-c485-4f26-afb5-a8f816128ab2',
                    'template_name'                             => 'SAP_TRFC_QRFC_CALLS_OUTBOUND_BLOCKED_QUEUES',
                    'name'                                      => 'SAP Outbound qRFC',
                    'container_id'                              => '1',
                    'servicetemplatetype_id'                    => '1',
                    'check_period_id'                           => '1',
                    'notify_period_id'                          => '1',
                    'description'                               => '',
                    'command_id'                                => 'c580ca9e-4181-4fcc-afe3-410e5fa1b3f9',
                    'check_command_args'                        => '',
                    'checkcommand_info'                         => '',
                    'eventhandler_command_id'                   => '0',
                    'timeperiod_id'                             => null,
                    'check_interval'                            => '120',
                    'retry_interval'                            => '60',
                    'max_check_attempts'                        => '3',
                    'first_notification_delay'                  => '0',
                    'notification_interval'                     => '7200',
                    'notify_on_warning'                         => '1',
                    'notify_on_unknown'                         => '1',
                    'notify_on_critical'                        => '1',
                    'notify_on_recovery'                        => '1',
                    'notify_on_flapping'                        => '0',
                    'notify_on_downtime'                        => '0',
                    'flap_detection_enabled'                    => '0',
                    'flap_detection_on_ok'                      => '0',
                    'flap_detection_on_warning'                 => '0',
                    'flap_detection_on_unknown'                 => '0',
                    'flap_detection_on_critical'                => '0',
                    'low_flap_threshold'                        => '0',
                    'high_flap_threshold'                       => '0',
                    'process_performance_data'                  => '1',
                    'freshness_checks_enabled'                  => '0',
                    'freshness_threshold'                       => '3600',
                    'passive_checks_enabled'                    => '1',
                    'event_handler_enabled'                     => '0',
                    'active_checks_enabled'                     => '1',
                    'retain_status_information'                 => '0',
                    'retain_nonstatus_information'              => '0',
                    'notifications_enabled'                     => '1',
                    'notes'                                     => '',
                    'priority'                                  => '1',
                    'tags'                                      => '',
                    'service_url'                               => '',
                    'is_volatile'                               => '0',
                    'check_freshness'                           => '0',
                    'created'                                   => '2021-03-25 14:17:48',
                    'modified'                                  => '2021-03-25 14:17:48',
                    'servicetemplateeventcommandargumentvalues' =>
                        [
                        ],
                    'servicetemplatecommandargumentvalues'      =>
                        [
                            0 =>
                                [
                                    'commandargument_id' => '$ARG1$',
                                    'value'              => 'SAP CCMS Technical Expert Monitors',
                                ],
                            1 =>
                                [
                                    'commandargument_id' => '$ARG2$',
                                    'value'              => 'All Monitoring Contexts',
                                ],
                            2 =>
                                [
                                    'commandargument_id' => '$ARG3$',
                                    'value'              => 'Transactional RFC and Queued RFC#.*Outbound.*#Blocked queues: Client $_HOSTCLIENT$',
                                ],
                            3 =>
                                [
                                    'commandargument_id' => '$ARG4$',
                                    'value'              => '',
                                ],
                            4 =>
                                [
                                    'commandargument_id' => '$ARG5$',
                                    'value'              => '',
                                ],
                        ],
                    'customvariables'                           =>
                        [
                        ],
                    'servicegroups'                             =>
                        [
                        ],
                    'contactgroups'                             =>
                        [
                        ],
                    'contacts'                                  =>
                        [
                        ],
                ],
        ];
        return $data;
    }

    public function getServicetemplategroupsData() {
        $data = [
            0 =>
                [
                    'uuid'             => '487d4f3e-2455-4c94-a600-593e5867bf5f',
                    'description'      => '',
                    'servicetemplates' =>
                        [
                            0  =>
                                [
                                    'id'   => '0a362c3d-492d-4a70-835c-5e906abe8fd8',
                                    'uuid' => '0a362c3d-492d-4a70-835c-5e906abe8fd8',
                                ],
                            1  =>
                                [
                                    'id'   => 'bb78bb0a-8f73-4872-9df3-f04a17ef87c4',
                                    'uuid' => 'bb78bb0a-8f73-4872-9df3-f04a17ef87c4',
                                ],
                            2  =>
                                [
                                    'id'   => '7c80700f-cec6-453e-8620-c5e27216fb77',
                                    'uuid' => '7c80700f-cec6-453e-8620-c5e27216fb77',
                                ],
                            3  =>
                                [
                                    'id'   => 'ded51ee1-62aa-4e9b-a75e-7f829eaff9e5',
                                    'uuid' => 'ded51ee1-62aa-4e9b-a75e-7f829eaff9e5',
                                ],
                            4  =>
                                [
                                    'id'   => '68ad675c-389e-4a43-b823-06452c2e7ab9',
                                    'uuid' => '68ad675c-389e-4a43-b823-06452c2e7ab9',
                                ],
                            5  =>
                                [
                                    'id'   => 'ece23947-1298-4bc8-811e-b935475c8509',
                                    'uuid' => 'ece23947-1298-4bc8-811e-b935475c8509',
                                ],
                            6  =>
                                [
                                    'id'   => '0239aed8-11a5-4aaf-9464-285c5e97fc50',
                                    'uuid' => '0239aed8-11a5-4aaf-9464-285c5e97fc50',
                                ],
                            7  =>
                                [
                                    'id'   => 'bdd3e3bf-58fe-4979-bc40-49982bb5f0e0',
                                    'uuid' => 'bdd3e3bf-58fe-4979-bc40-49982bb5f0e0',
                                ],
                            8  =>
                                [
                                    'id'   => '27fc931b-2e70-4b99-a58f-f7080ea5ccdd',
                                    'uuid' => '27fc931b-2e70-4b99-a58f-f7080ea5ccdd',
                                ],
                            9  =>
                                [
                                    'id'   => '4f51d6a0-3b27-4cf7-b2c0-5b282fe425d3',
                                    'uuid' => '4f51d6a0-3b27-4cf7-b2c0-5b282fe425d3',
                                ],
                            10 =>
                                [
                                    'id'   => '98aca158-a75a-4ae4-9b4f-469bd0fb86f0',
                                    'uuid' => '98aca158-a75a-4ae4-9b4f-469bd0fb86f0',
                                ],
                            11 =>
                                [
                                    'id'   => '761c0350-307b-4ae5-986f-13f153ecfcca',
                                    'uuid' => '761c0350-307b-4ae5-986f-13f153ecfcca',
                                ],
                            12 =>
                                [
                                    'id'   => 'c1a248c2-9e2e-4107-8de9-2003f98fc643',
                                    'uuid' => 'c1a248c2-9e2e-4107-8de9-2003f98fc643',
                                ],
                            13 =>
                                [
                                    'id'   => '8cc59b1f-37e6-4db0-9836-75a77e6add73',
                                    'uuid' => '8cc59b1f-37e6-4db0-9836-75a77e6add73',
                                ],
                            14 =>
                                [
                                    'id'   => '88f71a5c-6e28-47f9-8187-65816e27baf5',
                                    'uuid' => '88f71a5c-6e28-47f9-8187-65816e27baf5',
                                ],
                            15 =>
                                [
                                    'id'   => 'eebd9d2e-b05b-4715-8e96-5979a969e83e',
                                    'uuid' => 'eebd9d2e-b05b-4715-8e96-5979a969e83e',
                                ],
                            16 =>
                                [
                                    'id'   => '1ba438e8-7947-4f8e-bc1f-2738c379c7fc',
                                    'uuid' => '1ba438e8-7947-4f8e-bc1f-2738c379c7fc',
                                ],
                            17 =>
                                [
                                    'id'   => 'c8c5ae1d-4df2-4c6f-adf4-dbaa02cb5d76',
                                    'uuid' => 'c8c5ae1d-4df2-4c6f-adf4-dbaa02cb5d76',
                                ],
                            18 =>
                                [
                                    'id'   => '54179f08-2469-4f66-9806-510c42314f75',
                                    'uuid' => '54179f08-2469-4f66-9806-510c42314f75',
                                ],
                            19 =>
                                [
                                    'id'   => 'a5949e1c-0911-4dea-a58d-081cb54354bf',
                                    'uuid' => 'a5949e1c-0911-4dea-a58d-081cb54354bf',
                                ],
                            20 =>
                                [
                                    'id'   => 'b77ee087-4746-4293-b2ed-93469e12463c',
                                    'uuid' => 'b77ee087-4746-4293-b2ed-93469e12463c',
                                ],
                            21 =>
                                [
                                    'id'   => '3688e74e-6e91-4a78-97f0-1fd05e12d14d',
                                    'uuid' => '3688e74e-6e91-4a78-97f0-1fd05e12d14d',
                                ],
                            22 =>
                                [
                                    'id'   => 'b2a9c408-6453-4045-8e20-3f5b7efbf084',
                                    'uuid' => 'b2a9c408-6453-4045-8e20-3f5b7efbf084',
                                ],
                            23 =>
                                [
                                    'id'   => '1960f0be-f1bc-4c93-b772-090c8f458d58',
                                    'uuid' => '1960f0be-f1bc-4c93-b772-090c8f458d58',
                                ],
                            24 =>
                                [
                                    'id'   => '257a88e8-645b-4a67-8cb7-ac3fc6659197',
                                    'uuid' => '257a88e8-645b-4a67-8cb7-ac3fc6659197',
                                ],
                            25 =>
                                [
                                    'id'   => 'da7ab213-b175-4719-ad89-fe15316af3a3',
                                    'uuid' => 'da7ab213-b175-4719-ad89-fe15316af3a3',
                                ],
                            26 =>
                                [
                                    'id'   => '30293841-24eb-41d9-81c6-f0ffb00c3646',
                                    'uuid' => '30293841-24eb-41d9-81c6-f0ffb00c3646',
                                ],
                            27 =>
                                [
                                    'id'   => '5cb52931-3197-43dd-8454-5ccb6d3a7c8c',
                                    'uuid' => '5cb52931-3197-43dd-8454-5ccb6d3a7c8c',
                                ],
                            28 =>
                                [
                                    'id'   => '4479a0d2-ffae-44b9-b928-4d989898bce7',
                                    'uuid' => '4479a0d2-ffae-44b9-b928-4d989898bce7',
                                ],
                            29 =>
                                [
                                    'id'   => '7288efda-7dbe-4119-ade8-8038151519de',
                                    'uuid' => '7288efda-7dbe-4119-ade8-8038151519de',
                                ],
                            30 =>
                                [
                                    'id'   => 'a788168e-5da2-4991-a50b-4ec03af75f2c',
                                    'uuid' => 'a788168e-5da2-4991-a50b-4ec03af75f2c',
                                ],
                            31 =>
                                [
                                    'id'   => 'd7f66176-04d1-4316-8a72-396e8011a199',
                                    'uuid' => 'd7f66176-04d1-4316-8a72-396e8011a199',
                                ],
                            32 =>
                                [
                                    'id'   => '32cf14dc-060e-48cb-a4ad-b51a34b26dae',
                                    'uuid' => '32cf14dc-060e-48cb-a4ad-b51a34b26dae',
                                ],
                            33 =>
                                [
                                    'id'   => '6fe8cbb0-8234-415c-aabb-723d11b5ecab',
                                    'uuid' => '6fe8cbb0-8234-415c-aabb-723d11b5ecab',
                                ],
                            34 =>
                                [
                                    'id'   => '63e968db-61c5-452b-a498-6c65cee164e0',
                                    'uuid' => '63e968db-61c5-452b-a498-6c65cee164e0',
                                ],
                            35 =>
                                [
                                    'id'   => '64915c2c-5b83-468a-a99c-103e966ecc6e',
                                    'uuid' => '64915c2c-5b83-468a-a99c-103e966ecc6e',
                                ],
                            36 =>
                                [
                                    'id'   => 'de406bf3-049d-4498-95fe-0936b2c75a20',
                                    'uuid' => 'de406bf3-049d-4498-95fe-0936b2c75a20',
                                ],
                            37 =>
                                [
                                    'id'   => '7f6cdf7f-6471-4f51-843f-560a91485553',
                                    'uuid' => '7f6cdf7f-6471-4f51-843f-560a91485553',
                                ],
                            38 =>
                                [
                                    'id'   => '6094ee60-e6b7-4158-afc7-78d1160ded84',
                                    'uuid' => '6094ee60-e6b7-4158-afc7-78d1160ded84',
                                ],
                            39 =>
                                [
                                    'id'   => 'a78f65c2-722c-428f-806c-4caf9cee9655',
                                    'uuid' => 'a78f65c2-722c-428f-806c-4caf9cee9655',
                                ],
                            40 =>
                                [
                                    'id'   => 'ec121498-9e16-4245-9937-b95864d41c07',
                                    'uuid' => 'ec121498-9e16-4245-9937-b95864d41c07',
                                ],
                            41 =>
                                [
                                    'id'   => '324b5bf2-54da-4be5-bb40-2bec79b91ada',
                                    'uuid' => '324b5bf2-54da-4be5-bb40-2bec79b91ada',
                                ],
                            42 =>
                                [
                                    'id'   => '2de342c4-56bc-4985-b63e-206de62c6aef',
                                    'uuid' => '2de342c4-56bc-4985-b63e-206de62c6aef',
                                ],
                            43 =>
                                [
                                    'id'   => 'b16d0a14-bb21-4f8f-abb0-ef0234f890fe',
                                    'uuid' => 'b16d0a14-bb21-4f8f-abb0-ef0234f890fe',
                                ],
                            44 =>
                                [
                                    'id'   => 'f3dc4237-2034-4629-9df5-422735de9fd9',
                                    'uuid' => 'f3dc4237-2034-4629-9df5-422735de9fd9',
                                ],
                            45 =>
                                [
                                    'id'   => '3e53f345-3aef-4cb1-9bba-f755ab22cb77',
                                    'uuid' => '3e53f345-3aef-4cb1-9bba-f755ab22cb77',
                                ],
                            46 =>
                                [
                                    'id'   => 'd80e9b1e-d1a0-4531-a39a-91677ceb4076',
                                    'uuid' => 'd80e9b1e-d1a0-4531-a39a-91677ceb4076',
                                ],
                            47 =>
                                [
                                    'id'   => '31206347-a022-42e8-b427-fb2c83ad2126',
                                    'uuid' => '31206347-a022-42e8-b427-fb2c83ad2126',
                                ],
                            48 =>
                                [
                                    'id'   => '4fba0c56-8903-4bcc-897a-c4e36d1191a0',
                                    'uuid' => '4fba0c56-8903-4bcc-897a-c4e36d1191a0',
                                ],
                            49 =>
                                [
                                    'id'   => '6f0023a9-e35f-4805-aa4a-f7916c434552',
                                    'uuid' => '6f0023a9-e35f-4805-aa4a-f7916c434552',
                                ],
                            50 =>
                                [
                                    'id'   => '5fd3066b-5927-4a77-b626-f0e8812827c5',
                                    'uuid' => '5fd3066b-5927-4a77-b626-f0e8812827c5',
                                ],
                            51 =>
                                [
                                    'id'   => '7aacb951-6bb3-444f-bccf-1106701873ad',
                                    'uuid' => '7aacb951-6bb3-444f-bccf-1106701873ad',
                                ],
                            52 =>
                                [
                                    'id'   => '2bcf28e8-bdf8-40aa-8732-7bbb6789803a',
                                    'uuid' => '2bcf28e8-bdf8-40aa-8732-7bbb6789803a',
                                ],
                            53 =>
                                [
                                    'id'   => '6ef4882c-1e32-4935-9345-696b80da3081',
                                    'uuid' => '6ef4882c-1e32-4935-9345-696b80da3081',
                                ],
                            54 =>
                                [
                                    'id'   => '87121e19-6fc0-45cc-8588-23dcf4553f1a',
                                    'uuid' => '87121e19-6fc0-45cc-8588-23dcf4553f1a',
                                ],
                            55 =>
                                [
                                    'id'   => '03d28e08-1109-4032-8708-34ee2aff0040',
                                    'uuid' => '03d28e08-1109-4032-8708-34ee2aff0040',
                                ],
                            56 =>
                                [
                                    'id'   => '4eae2911-1371-4e61-b6d1-61139748cb08',
                                    'uuid' => '4eae2911-1371-4e61-b6d1-61139748cb08',
                                ],
                            57 =>
                                [
                                    'id'   => 'dddb8f26-1058-4bd5-80b1-38beb0e61ced',
                                    'uuid' => 'dddb8f26-1058-4bd5-80b1-38beb0e61ced',
                                ],
                            58 =>
                                [
                                    'id'   => '619b14c4-2ce2-4161-b4be-6c4d5f37fbb7',
                                    'uuid' => '619b14c4-2ce2-4161-b4be-6c4d5f37fbb7',
                                ],
                            59 =>
                                [
                                    'id'   => 'aae243fa-e3d7-4978-9629-f6eb82295b08',
                                    'uuid' => 'aae243fa-e3d7-4978-9629-f6eb82295b08',
                                ],
                            60 =>
                                [
                                    'id'   => '19ce6706-ac6e-413f-9b1d-8abbdc2cd801',
                                    'uuid' => '19ce6706-ac6e-413f-9b1d-8abbdc2cd801',
                                ],
                            61 =>
                                [
                                    'id'   => 'fcc0153b-c05f-4d76-a707-898e77ee28d0',
                                    'uuid' => 'fcc0153b-c05f-4d76-a707-898e77ee28d0',
                                ],
                            62 =>
                                [
                                    'id'   => 'ffbd4262-0bc6-45c7-ad7d-5e1f8df00355',
                                    'uuid' => 'ffbd4262-0bc6-45c7-ad7d-5e1f8df00355',
                                ],
                            63 =>
                                [
                                    'id'   => 'b8dea6be-1049-4225-8dc6-2a34e4608db4',
                                    'uuid' => 'b8dea6be-1049-4225-8dc6-2a34e4608db4',
                                ],
                            64 =>
                                [
                                    'id'   => 'f8400b9a-7dbe-4724-9426-211bfe74c711',
                                    'uuid' => 'f8400b9a-7dbe-4724-9426-211bfe74c711',
                                ],
                            65 =>
                                [
                                    'id'   => 'ca3fd6e5-11e5-4991-b036-64107a81c258',
                                    'uuid' => 'ca3fd6e5-11e5-4991-b036-64107a81c258',
                                ],
                            66 =>
                                [
                                    'id'   => '871cb131-d3a2-43fa-960e-338e8d05fa59',
                                    'uuid' => '871cb131-d3a2-43fa-960e-338e8d05fa59',
                                ],
                            67 =>
                                [
                                    'id'   => '79ade959-5193-490b-b624-9f7e3503c529',
                                    'uuid' => '79ade959-5193-490b-b624-9f7e3503c529',
                                ],
                            68 =>
                                [
                                    'id'   => 'b70afa65-a2a0-4423-b9ba-a857fb4b97ba',
                                    'uuid' => 'b70afa65-a2a0-4423-b9ba-a857fb4b97ba',
                                ],
                            69 =>
                                [
                                    'id'   => '0727f21b-6307-4d82-aa46-aa4bbc91ebdb',
                                    'uuid' => '0727f21b-6307-4d82-aa46-aa4bbc91ebdb',
                                ],
                            70 =>
                                [
                                    'id'   => 'e97caf28-50e5-42c0-aef1-789353b89aba',
                                    'uuid' => 'e97caf28-50e5-42c0-aef1-789353b89aba',
                                ],
                            71 =>
                                [
                                    'id'   => '8976fedf-96af-4b14-a571-9fd255503aec',
                                    'uuid' => '8976fedf-96af-4b14-a571-9fd255503aec',
                                ],
                            72 =>
                                [
                                    'id'   => 'ea318c60-92bc-418f-b3a5-59687f200b3e',
                                    'uuid' => 'ea318c60-92bc-418f-b3a5-59687f200b3e',
                                ],
                            73 =>
                                [
                                    'id'   => '44c90da2-4f0d-4390-8c0f-f078d275810e',
                                    'uuid' => '44c90da2-4f0d-4390-8c0f-f078d275810e',
                                ],
                            74 =>
                                [
                                    'id'   => 'c58f0349-2d0e-4eec-9122-cdc0d5d6c9a0',
                                    'uuid' => 'c58f0349-2d0e-4eec-9122-cdc0d5d6c9a0',
                                ],
                            75 =>
                                [
                                    'id'   => 'a50fe427-42e0-49d0-89a1-a0f40a1b33ea',
                                    'uuid' => 'a50fe427-42e0-49d0-89a1-a0f40a1b33ea',
                                ],
                            76 =>
                                [
                                    'id'   => '6560c534-c485-4f26-afb5-a8f816128ab2',
                                    'uuid' => '6560c534-c485-4f26-afb5-a8f816128ab2',
                                ],
                        ],
                    'container'        =>
                        [
                            'name'             => 'SAP Monitoring',
                            'parent_id'        => 1,
                            'containertype_id' => '9',
                        ],
                ],
        ];
        return $data;
    }

    /**
     * @param string $name
     * @param array $commandarguments
     * @return bool|mixed
     */
    public function getCommandArgumentIdByName(string $name, array $commandarguments) {
        foreach ($commandarguments as $commandargument) {
            if (isset($commandargument['name']) && $commandargument['name'] === $name) {
                return $commandargument['id'];
            }
        }
        return false;
    }

    /**
     * @return array
     */
    public function getWizardAssignmentData() {
        $data = [
            'uuid'             => '82d5f14e-87ec-4da3-9c09-f42032ab5326',
            'type_id'          => 'sap',
            'servicetemplates' => [
                '_ids' => [
                    '0a362c3d-492d-4a70-835c-5e906abe8fd8',
                    'bb78bb0a-8f73-4872-9df3-f04a17ef87c4',
                    '7c80700f-cec6-453e-8620-c5e27216fb77',
                    'ded51ee1-62aa-4e9b-a75e-7f829eaff9e5',
                    '68ad675c-389e-4a43-b823-06452c2e7ab9',
                    'ece23947-1298-4bc8-811e-b935475c8509',
                    '0239aed8-11a5-4aaf-9464-285c5e97fc50',
                    'bdd3e3bf-58fe-4979-bc40-49982bb5f0e0',
                    '27fc931b-2e70-4b99-a58f-f7080ea5ccdd',
                    '4f51d6a0-3b27-4cf7-b2c0-5b282fe425d3',
                    '98aca158-a75a-4ae4-9b4f-469bd0fb86f0',
                    '761c0350-307b-4ae5-986f-13f153ecfcca',
                    'c1a248c2-9e2e-4107-8de9-2003f98fc643',
                    '8cc59b1f-37e6-4db0-9836-75a77e6add73',
                    '88f71a5c-6e28-47f9-8187-65816e27baf5',
                    'eebd9d2e-b05b-4715-8e96-5979a969e83e',
                    '1ba438e8-7947-4f8e-bc1f-2738c379c7fc',
                    'c8c5ae1d-4df2-4c6f-adf4-dbaa02cb5d76',
                    '54179f08-2469-4f66-9806-510c42314f75',
                    'a5949e1c-0911-4dea-a58d-081cb54354bf',
                    'b77ee087-4746-4293-b2ed-93469e12463c',
                    '3688e74e-6e91-4a78-97f0-1fd05e12d14d',
                    'b2a9c408-6453-4045-8e20-3f5b7efbf084',
                    '1960f0be-f1bc-4c93-b772-090c8f458d58',
                    '257a88e8-645b-4a67-8cb7-ac3fc6659197',
                    'da7ab213-b175-4719-ad89-fe15316af3a3',
                    '30293841-24eb-41d9-81c6-f0ffb00c3646',
                    '5cb52931-3197-43dd-8454-5ccb6d3a7c8c',
                    '4479a0d2-ffae-44b9-b928-4d989898bce7',
                    '7288efda-7dbe-4119-ade8-8038151519de',
                    'a788168e-5da2-4991-a50b-4ec03af75f2c',
                    'd7f66176-04d1-4316-8a72-396e8011a199',
                    '32cf14dc-060e-48cb-a4ad-b51a34b26dae',
                    '6fe8cbb0-8234-415c-aabb-723d11b5ecab',
                    '63e968db-61c5-452b-a498-6c65cee164e0',
                    '64915c2c-5b83-468a-a99c-103e966ecc6e',
                    'de406bf3-049d-4498-95fe-0936b2c75a20',
                    '7f6cdf7f-6471-4f51-843f-560a91485553',
                    '6094ee60-e6b7-4158-afc7-78d1160ded84',
                    'a78f65c2-722c-428f-806c-4caf9cee9655',
                    'ec121498-9e16-4245-9937-b95864d41c07',
                    '324b5bf2-54da-4be5-bb40-2bec79b91ada',
                    '2de342c4-56bc-4985-b63e-206de62c6aef',
                    'b16d0a14-bb21-4f8f-abb0-ef0234f890fe',
                    'f3dc4237-2034-4629-9df5-422735de9fd9',
                    '3e53f345-3aef-4cb1-9bba-f755ab22cb77',
                    'd80e9b1e-d1a0-4531-a39a-91677ceb4076',
                    '31206347-a022-42e8-b427-fb2c83ad2126',
                    '4fba0c56-8903-4bcc-897a-c4e36d1191a0',
                    '6f0023a9-e35f-4805-aa4a-f7916c434552',
                    '5fd3066b-5927-4a77-b626-f0e8812827c5',
                    '7aacb951-6bb3-444f-bccf-1106701873ad',
                    '2bcf28e8-bdf8-40aa-8732-7bbb6789803a',
                    '6ef4882c-1e32-4935-9345-696b80da3081',
                    '87121e19-6fc0-45cc-8588-23dcf4553f1a',
                    '03d28e08-1109-4032-8708-34ee2aff0040',
                    '4eae2911-1371-4e61-b6d1-61139748cb08',
                    'dddb8f26-1058-4bd5-80b1-38beb0e61ced',
                    '619b14c4-2ce2-4161-b4be-6c4d5f37fbb7',
                    'aae243fa-e3d7-4978-9629-f6eb82295b08',
                    '19ce6706-ac6e-413f-9b1d-8abbdc2cd801',
                    'fcc0153b-c05f-4d76-a707-898e77ee28d0',
                    'ffbd4262-0bc6-45c7-ad7d-5e1f8df00355',
                    'b8dea6be-1049-4225-8dc6-2a34e4608db4',
                    'f8400b9a-7dbe-4724-9426-211bfe74c711',
                    'ca3fd6e5-11e5-4991-b036-64107a81c258',
                    '871cb131-d3a2-43fa-960e-338e8d05fa59',
                    '79ade959-5193-490b-b624-9f7e3503c529',
                    'b70afa65-a2a0-4423-b9ba-a857fb4b97ba',
                    '0727f21b-6307-4d82-aa46-aa4bbc91ebdb',
                    'e97caf28-50e5-42c0-aef1-789353b89aba',
                    '8976fedf-96af-4b14-a571-9fd255503aec',
                    'ea318c60-92bc-418f-b3a5-59687f200b3e',
                    '44c90da2-4f0d-4390-8c0f-f078d275810e',
                    'c58f0349-2d0e-4eec-9122-cdc0d5d6c9a0',
                    'a50fe427-42e0-49d0-89a1-a0f40a1b33ea',
                    '6560c534-c485-4f26-afb5-a8f816128ab2'
                ]
            ]
        ];
        return $data;
    }
}
